;Licence: Attribution-NonCommercial-ShareAlike
;Written by: Sean Boyce on April 01, 2022
;Contact: sean.a.boyce@gmail.com
;https://creativecommons.org
;Written for Attiny12L clocked up to 4Mhz. The Quantum Event is a falling edge, make sure you use an even number of inversion stages! Otherwise you may be measuring event duration, not time between events.
;This code should work well on most Atmel MCUs, just update timer and pin registers as needed and relace the include file above.
;CODE BEGINS

;We use an ATTiny12, so include the .def file from the manufacturer to make life easier.
.include "tn12def.inc"

;Define reset vector

.cseg
	.org 0x00
	RJMP reset
	
;Set up registers

reset:
;R16 is general purpose temp register
CLR r16
;R19 tracks whether we are t0 or t1
CLR r19
;R20 tracks t0 time value
CLR r20
;R21 tracks t1 time value
CLR r21
;R22 is always zero
CLR r22

;Set PB3 and PB4 as input. PB3 as trigger, PB4 as Quantum Event input. PB0 and PB1 are output.
CLR r16
LDI r16, 0b00000011
OUT DDRB, r16

;Setup timer at full speed
LDI r16, (0 << CS02) | (0 << CS01) | (1 << CS00)
OUT TCCR0, r16
CLR r16


;ALGORITHM BEGINS
;In short, we measure 2 time intervals between Quantum Events. If the first interval is bigger, one output line is raised HIGH. If the second interval is bigger, the other output line is raised high. Algorithm developed by John Von Neumann in 1951. Citation follows.   
;J. Von Neumann, “Various techniques used in connection with random digits,” Applied Math Series, Notes by G. E. Forsythe, in National Bureau of Standards, Vol. 12, pp. 36-38, 1951. 

start:
;If PB3 is HIGH at start, just wait! We've received a trigger pulse and the pulse is still high.
SBIC PINB,3
RJMP start

;If PB3 is low at start, make sure output is zero. Then wait for rising edge.
;IN r16, PINB
;ANDI r16, 0b00011000
CBI PORTB, 0
CBI PORTB, 1
wait_for_trigger:
SBIS PINB,3
RJMP wait_for_trigger

t0: ;Get outselves a fresh start before sampling
SBIS PINB, 4
RJMP t0

sample1: ;Get outselves a fresh start before sampling
SBIC PINB, 4
RJMP sample1
OUT TCNT0, r22 ; clear timer to begin sampling

sample2: 
SBIS PINB, 4 ;Wait for pin D2 to go high
RJMP sample2

sample3: 
SBIC PINB, 4 ;Wait for pin D2 to go low. Done sampling t0!
RJMP sample3
IN r20, TCNT0

t1: ;Get outselves a fresh start before sampling.
SBIS PINB, 4
RJMP t1

sample4: ;Get outselves a fresh start before sampling
SBIC PINB, 4
RJMP sample4
OUT TCNT0, r22 ; clear timer to begin sampling

sample5: 
SBIS PINB, 4 ;Wait for pin D2 to go high
RJMP sample5

sample6: 
SBIC PINB, 4 ;Wait for pin D2 to go low. Done sampling t1!
RJMP sample6
IN r21, TCNT0

;If t0>t1, waveform collapses to state 0. If t0<t1, waveform collapses to state 1. If equal, skip this round and try again. 


CP r20, r21 ; If exactly equal, cancel this round and try again
BREQ t0
CLC
CP r20, r21
BRSH one
;State zero. Confirm trigger is still high, then output and restart.
SBIS PINB,3
RJMP start
SBI PORTB, 0
CLR r16
CLR r19
CLR r20
CLR r21
RJMP start

;State one. Confirm trigger is still high, then output and restart.
one:
SBIS PINB,3
RJMP start
SBI PORTB, 1
CLR r16
CLR r19
CLR r20
CLR r21
RJMP start





